import pymysql

def __get_mysql_connection_with_config(config):
    '''
    通过标准配置连接数据库，底层方法
    :param config: 字典类型，形如：{'host': '127.0.0.1', 'port': 3306, 'user': 'test', 'password': 'test', 'database': 'users'}
    :return: Connection类型
    '''
    if config.get('charset') is None:
        config['charset'] = 'utf8'
    connect = pymysql.connect(**config)
    return connect

def __query_one(con,sql):
    cur = con.cursor()
    cur.execute(sql)
    result = cur.fetchone()
    fields_list = []
    for field in cur.description:
        fields_list.append(field[0])
    con.commit()
    result_dict = {}
    if result is None or len(result) == 0:
        return None
    for index in range(len(result)):
        result_dict[fields_list[index]] = result[index]
    cur.close()
    con.close()
    return result_dict

def __query_all(con,sql):
    cur = con.cursor()
    cur.execute(sql)
    result = cur.fetchall()
    fields_list = []
    for field in cur.description:
        fields_list.append(field[0])
    con.commit()
    if result is None or len(result) == 0:
        return None
    result_list = []
    for i in range(len(result)):
        row_dict = {}
        row = result[i]
        for j in range(len(row)):
            row_dict[fields_list[j]] = row[j]
        result_list.append(row_dict)
    cur.close()
    con.close()
    return result_list

def __execute_sql(con,sql):
    cur = con.cursor()
    cur.execute(sql)
    con.commit()
    cur.close()
    con.close()

def excute_mysql_sql_with_mysql(config,sql):
    '''
    数据库的删除、修改调用此方法
    :param config: 字典类型，形如：{'host': '127.0.0.1', 'port': 3306, 'user': 'test', 'password': 'test', 'database': 'users'}
    :param sql: sql语句
    :return: 空
    '''
    con = __get_mysql_connection_with_config(config)
    return __execute_sql(con,sql)

def query_one_with_config(config,sql):
    '''
    查询一条数据调用此方法，如果有多条数据，查询出来的是最老的一条
    :param config: 字典类型，形如：{'host': '127.0.0.1', 'port': 3306, 'user': 'test', 'password': 'test', 'database': 'users'}
    :param sql: sql语句
    :return: {'':''} 字典数据格式
    '''
    con = __get_mysql_connection_with_config(config)
    return __query_one(con,sql)

def query_all_with_config(config,sql):
    '''
    查询多条数据调用此方法
    :param config: 字典类型，形如：{'host': '127.0.0.1', 'port': 3306, 'user': 'test', 'password': 'test', 'database': 'users'}
    :param sql: sql语句
    :return:[{'':''},{'':''}] 列表数据格式
    '''
    con = __get_mysql_connection_with_config(config)
    return __query_all(con, sql)

def get_define_data_by_field(config,sql,field):
    '''
    先查询出所有数据，再根据传入的field返回该字段对应的所有值，例如一个表有多条订单，传入订单号字段返回所有订单号的LIST
    :param config: config: 字典类型，形如：{'host': '127.0.0.1', 'port': 3306, 'user': 'test', 'password': 'test', 'database': 'users'}
    :param sql: sql: sql语句
    :param field: 想要获取的字段，例如order_no
    :return: [] 列表数据类型
    '''
    database_data = query_all_with_config(config, sql)
    field_list = []
    try:
        for i in range(len(database_data)):
            if field not in database_data[i].keys():
                print('field【%s】不存在，请检查！' % field)
                break
            else:
                field_list.append(database_data[i].get(field))
        return field_list
    except TypeError as e:
        return field_list
    # except

def get_define_data_by_database(config,sql,rownum=0):
    '''
    先查询所有数据，再根据rownum，查询某一行
    :param config: config: 字典类型，形如：{'host': '127.0.0.1', 'port': 3306, 'user': 'test', 'password': 'test', 'database': 'users'}
    :param sql: sql: sql语句
    :param rownum: 数字格式，想要查询的行，从0开始
    :return: {'':''} 字典数据格式
    '''
    database_data = query_all_with_config(config,sql)
    return database_data[rownum]


from decimal import Decimal

from prettytable import PrettyTable
import copy
from deal_float import float_deal  # Decimal格式

class StagePlanUtils:
    def __init__(self,principal,annual_rate):
        self.principal = Decimal(principal)  # 本金
        self.annual_rate = Decimal(annual_rate)  # 年化利率
        self.term_rate =  self.annual_rate / Decimal('12') # 月利率
        self.day_rate = self.annual_rate / Decimal('12') / Decimal('30')  # 日利率
        self.line = PrettyTable(["期数", "还款本息", "当期本金", "当期利息", "剩余本金", "还款方式"])

    repayPattern = {
        "MCAT": "随借随还,按日计息",
        "BFTO": "利随本清,按日计息",
        "MIRD": "按月付息到期还本",
        "EPEI": "等本等息,按月付息",  # 对借款人最不利
        "MCEP": "等额本金,按月付息",  # 借款总利息最低
        "MCEI": "等额等息,按月付息"
    }


    # line.align["期数"] = "1"  # 以期数字段左对齐
    # line.padding_width = 2    # 填充宽度(字段间隔)


    # "EPEI": "等本等息,按月付息"               每期利息和本金一样,最后一期本金为上一期的剩余本金，最后一期利息为总利息-已还利息
    # def get_EPEI(num):
    #     show = copy.deepcopy(line)
    #     term_int = float_deal(principal * term_rate)  # 每期还利息（2位小数）
    #     total_int = float_deal(principal * term_rate * num)  # 总利息（2位小数）
    #     term_prin = float_deal(principal / num)  # 每期还本金（2位小数）
    #     prin = float_deal(principal)  # 初始化剩余本金为借款本金
    #     for i in range(1, num + 1):
    #         # 如果是最后一期，还款本金为上一期的剩余本金; 利息为总利息-已还利息
    #         if i == num:
    #             term_prin = prin
    #             term_int = total_int - (term_int * (num - 1))
    #         term_amt = term_prin + term_int
    #         prin = prin - term_prin  # 剩余本金=上期剩余本金-当期还本金
    #         show.add_row([i, term_amt, term_prin, term_int, prin, 'EPEI'])
    #     print('等本等息总利息为：', total_int)
    #     print(show)


    # "MIRD": "按月付息到期还本"      最后一期还本金，每期利息固定
    # def get_MIRD(num):
    #     show = copy.deepcopy(line)
    #     term_int = float_deal(principal * term_rate)
    #     term_prin = float_deal(0.0)
    #     total_int = float_deal(principal * term_rate * num)  # 总利息（2位小数）
    #     prin = float_deal(principal)
    #     for i in range(1, num + 1):
    #         # 如果是最后一期，还款本金为上一期的剩余本金; 利息为总利息-已还利息
    #         if i == num:
    #             term_prin = prin
    #             term_int = total_int - (term_int * (num - 1))
    #         term_amt = term_prin + term_int
    #         prin = prin - term_prin  # 剩余本金=上期剩余本金-当期还本金
    #         show.add_row([i, term_amt, term_prin, term_int, prin, 'MIRD'])
    #     print('按月付息到期还本总利息为：', total_int)
    #     print(show)


    # "等额本金,按月付息"
    # def get_MCEP(num):
    #     ## 每月本金相同，利息递减，相当于剩余本金的利息,每期利息固定：上一期本金*利率
    #     show = copy.deepcopy(line)
    #     term_prin = float_deal(principal / num)  # 每期还本金（2位小数）
    #     prin = float_deal(principal)
    #     total_int = float_deal(0.0)
    #     for i in range(1, num + 1):
    #         term_int = float_deal(prin * float_deal(term_rate))  # 每期利息固定：上一期本金*利率
    #         # 如果是最后一期，还款本金为上一期的剩余本金;
    #         if i == num:
    #             term_prin = prin
    #         term_amt = term_prin + term_int
    #         prin = prin - term_prin  # 剩余本金=上期剩余本金-当期还本金
    #         show.add_row([i, term_amt, term_prin, term_int, prin, 'MCEP'])
    #         total_int = float_deal(total_int + term_int)
    #     print('等额本金总利息为：', total_int)
    #     print(show)

    # "MCEI": "等额本息,按月付息"
    def get_MCEI(self,num):
        # 本金+利息保持相同，本金逐月递增，利息逐月递减，月还款数不变。
        plan_list = []
        show = copy.deepcopy(self.line)
        term_amt = float_deal((self.principal*self.term_rate*(1+self.term_rate)**num)/((1+self.term_rate)**num-1))      # 每期还款总额       **是幂运算
        prin = float_deal(self.principal)
        total_int = float_deal(0.0)
        for i in range(1, num + 1):
            plan_by_list = {}
            term_int = float_deal(prin * float_deal(self.term_rate))        # 每期利息计算固定：上一期本金*利率
            term_prin = (term_amt - term_int)   # 如果是最后一期，还款本金为上一期的剩余本金;
            if i == num:
                term_prin = prin
            term_amt = term_prin + term_int
            prin = float_deal(prin - term_prin)      # 剩余本金=上期剩余本金-当期还本金
            show.add_row([i, term_amt, term_prin, term_int,prin,'MCEI'])
            plan_by_list[i] = term_amt*100, term_prin*100, term_int*100
            plan_list.append(plan_by_list)
            total_int = total_int +  term_int
        # print('等额本息总利息为：', total_int)
        print(show)
        print(plan_list)
        print(plan_list[1][2][2])
        return plan_list


    # # "MCAT": "随借随还,按日计息"
    # def get_MCAT(num, pay):
    #     show = copy.deepcopy(line)
    #     day_int = float_deal(pay * day_rate * num)
    #     repay_amt = day_int + pay
    #     re_prin = principal - pay
    #     if pay > principal:
    #         print("还款金额:", pay, "不能大于本金:", principal)
    #     else:
    #         show.add_row([1, repay_amt, pay, day_int, re_prin, 'MCAT'])
    #         print('随借随还总利息为：', day_int)
    #         print(show)
    #
    #
    # # "BFTO": "利随本清,按日计息"
    # def get_BFTO(num):
    #     show = copy.deepcopy(line)
    #     day_int = float_deal(principal * day_rate * num)
    #     repay_amt = day_int + principal
    #     show.add_row([1, repay_amt, principal, day_int, 0.0, 'BFTO'])
    #     print('利随本清总利息为：', day_int)
    #     print(show)



if __name__ == '__main__':
    # get_EPEI(3)  # 等本等息,按月付息
    # get_MIRD(3)  # 到期还本,按月付息
    # get_MCEP(3)  # 等额本金,按月付息
    StagePlanUtils('1000','0.16').get_MCEI(6)  # 等额本息,按月付息
    # get_MCAT(720, 195)  # 随借随还,按日计息



import requests
import json

################################
#封装的request和Response返回
_REQ="req"
_RESP="resp"
_URL="url"
_HEADER="header"
_PARAMS="params"
_METHOD="method"
_STATUS_CODE="status_code"
_BODY="body"

_METHOD_GET="GET"
_METHOD_POST="POST"
################################

session_req = requests.session()

def http_get(url, params=None, headers=None, timeout=None):
    if url is None:
        raise AssertionError('url参数是None!')
    try:
        response = session_req.get(url,headers=headers,params=params,timeout=timeout)
        # resAndRes = __get_req_and_resp_dict(params,headers,None,_METHOD_GET,response)
        Res = response.text
        resAndJosnDict = json.loads(Res)
        return resAndJosnDict
    except requests.exceptions.RequestException as e:
        raise AssertionError('发送请求失败，原因：%s'%e)

def http_post(url, params=None, headers=None, data=None,timeout=None):
    if url is None:
        raise AssertionError('url参数是None!')
    try:
        if headers is None:
            headers = {}
        headers.update({'Content-Type': 'application/json;charset=UTF-8'})
        if isinstance(data,str):
            data = json.loads(data)
        response = session_req.post(url,headers=headers,params=params,json=data,timeout=timeout)
        Res = response.text
        # resAndRes = __get_req_and_resp_dict(params,headers,data,_METHOD_POST,response)
        # resAndJosnStr = resAndRes.get('resp').get('body')
        resAndJosnDict = json.loads(Res)
        return resAndJosnDict
    except requests.exceptions.RequestException as e:
        raise AssertionError('发送请求失败，原因：%s'%e)

def __fill_req_dict(url,params,headers,body,method):
    req = {}
    req[_URL] = url
    req[_METHOD] = method
    req[_PARAMS] = params
    req[_HEADER] = headers
    body = str(body)
    req[body] = body
    return req

def __fill_resp_dict(response):
    resp = {}
    resp[_STATUS_CODE] = response.status_code
    resp[_HEADER] = response.headers
    resp[_BODY] = response.text
    return resp

def __get_req_and_resp_dict(params,headers,body,method,response):
    req_and_resp = {}
    url = response.url
    req = __fill_req_dict(url,params,headers,body,method)
    resp = __fill_resp_dict(response)
    req_and_resp[_REQ] = req
    req_and_resp[_RESP] = resp
    return req_and_resp



import configparser

class ConfigUtils:
    '''
    读取配置文件数据的底层方法
    '''
    def __init__(self, config_path):
        self.cfg = configparser.ConfigParser()
        self.cfg.read(config_path)

    def read_value(self,section,key):
        '''
        :param section: 配置中[]内的值
        :param key: 配置中值的key
        :return: 值的value
        Examples:
        参数 ConfigUtils(config_path).read_value('sit.mysql.users','database')
        返回 users  字符串数据类型
        '''
        value = self.cfg.get(section,key)
        return value

    def read_items(self,section):
        '''
        :param section: 配置中[]内的值
        :return: {'':'','':'','':'','':''} 字典数据类型
        Examples:
        参数 ConfigUtils(config_path).read_items('sit.mysql.users')
        '''
        items = self.cfg.items(section)
        dict = {}
        for i in range(len(items)):
            if items[i][0] != 'port':
                dict[items[i][0]] = items[i][1]
            else:
                dict[items[i][0]] = int(items[i][1])
        return dict


import random
import re
# 导入某个模块的部分类或方法
from datetime import datetime, timedelta

# 导入常量并重命名
from identity_random import constant as const


class IdNumber(str):

    def __init__(self, id_number):
        super(IdNumber, self).__init__()
        self.id = id_number
        self.area_id = int(self.id[0:6])
        self.birth_year = int(self.id[6:10])
        self.birth_month = int(self.id[10:12])
        self.birth_day = int(self.id[12:14])

    def get_area_name(self):
        """根据区域编号取出区域名称"""
        return const.AREA_INFO[self.area_id]

    def get_birthday(self):
        """通过身份证号获取出生日期"""
        return "{0}-{1}-{2}".format(self.birth_year, self.birth_month, self.birth_day)

    def get_age(self):
        """通过身份证号获取年龄"""
        now = (datetime.now() + timedelta(days=1))
        year, month, day = now.year, now.month, now.day

        if year == self.birth_year:
            return 0
        else:
            if self.birth_month > month or (self.birth_month == month and self.birth_day > day):
                return year - self.birth_year - 1
            else:
                return year - self.birth_year

    def get_sex(self):
        """通过身份证号获取性别， 女生：0，男生：1"""
        return int(self.id[16:17]) % 2

    def get_check_digit(self):
        """通过身份证号获取校验码"""
        check_sum = 0
        for i in range(0, 17):
            check_sum += ((1 << (17 - i)) % 11) * int(self.id[i])
        check_digit = (12 - (check_sum % 11)) % 11
        return check_digit if check_digit < 10 else 'X'

    @classmethod
    def verify_id(cls, id_number):
        """校验身份证是否正确"""
        if re.match(const.ID_NUMBER_18_REGEX, id_number):
            check_digit = cls(id_number).get_check_digit()
            return str(check_digit) == id_number[-1]
        else:
            return bool(re.match(const.ID_NUMBER_15_REGEX, id_number))

    @classmethod
    def generate_id(cls, sex=0, count=1):
        """随机生成身份证号，sex = 0表示女性，sex = 1表示男性"""
        random_id = []
        for i in range(count):
            # 随机生成一个区域码(6位数)
            id_number = str(random.choice(list(const.AREA_INFO.keys())))
            # 限定出生日期范围(8位数)
            start, end = datetime.strptime("1960-01-01", "%Y-%m-%d"), datetime.strptime("2019-12-30", "%Y-%m-%d")
            birth_days = datetime.strftime(start + timedelta(random.randint(0, (end - start).days + 1)), "%Y%m%d")
            id_number += str(birth_days)
            # 顺序码(2位数)
            id_number += str(random.randint(10, 99))
            # 性别码(1位数)
            id_number += str(random.randrange(sex, 10, step=2))
            # 校验码(1位数)
            random_id.append(id_number + str(cls(id_number).get_check_digit()))
        return random_id


if __name__ == '__main__':
    random_sex = random.randint(0, 1)  # 随机生成男(1)或女(0)
    print(IdNumber.generate_id(random_sex,3))  # 随机生成身份证号
    # print(IdNumber('410326199507103197').area_id)  # 地址编码:410326
    # print(IdNumber('410326199507103197').get_area_name())  # 地址:河南省洛阳市汝阳县
    # print(IdNumber('410326199507103197').get_birthday())  # 生日:1995-7-10
    # print(IdNumber('410326199507103197').get_age())  # 年龄:23(岁)
    # print(IdNumber('410326199507103197').get_sex())  # 性别:1(男)
    # print(IdNumber('410326199507103197').get_check_digit())  # 校验码:7
    # print(IdNumber.verify_id('410326199507103198'))  # 检验身份证是否正确:False